with 
  tests_data  as
  (
    select tst.*
    ,      tst.parameter_datatype||'_'||tst.return_datatype  as datatype_combo
    ,      case tst.test_type
             when 'PLAIN SQL'
             then 'SINGLE FUNCTION'
             else tst.test_type
           end                        as test_category
    from   ero_test_plsql_tests   tst
  )
  --
, tests_regular  as
  (
    select dat.*
    from   tests_data   dat
    where  dat.primary_function_type    = 'REGULAR'
      and  dat.secondary_function_type in ('NONE','REGULAR','DBMS')
  )
  --
, runtime_ratios as
  (
    select dat.test_type
    ,      dat.function_complexity
    ,      dat.primary_function_type
    ,      dat.secondary_function_type
    ,      dat.datatype_combo
    ,      round(dat.avg_runtime * 100/ reg.avg_runtime)             as ratio
    from   tests_data           dat
    join   tests_regular        reg
      on   reg.test_category        = dat.test_category
      and  reg.function_complexity  = dat.function_complexity
      and  reg.datatype_combo       = dat.datatype_combo
      and  (   dat.function_complexity <> 'COMPLEX'
            or case dat.secondary_function_type
                 when 'UDF' 
                 then 'REGULAR'
                 else dat.secondary_function_type
               end                      = reg.secondary_function_type
           )
  )
--
select *
from   runtime_ratios
pivot  ( 
         max(ratio)       -- aggregate is mandatory in pivot, but there's only one value, so max will do nicely
         for datatype_combo in ('NUMBER_NUMBER'                as NUM_NUM        
                               ,'NUMBER_VARCHAR2'              as NUM_VCHR2       
                               ,'NUMBER_DATE'                  as NUM_DATE       
                               ,'VARCHAR2_NUMBER'              as VCHR2_NUM      
                               ,'VARCHAR2_VARCHAR2'            as VCHR2_VCHR2    
                               ,'VARCHAR2_DATE'                as VCHR2_DATE     
                               ,'DATE_NUMBER'                  as DATE_NUM       
                               ,'DATE_VARCHAR2'                as DATE_VCHR2     
                               ,'DATE_DATE'                    as DATE_DATE      
                               ,'BINARY_FLOAT_BINARY_FLOAT'    as BIN_FLT_BIN_FLT
                               ,'BINARY_DOUBLE_BINARY_DOUBLE'  as BIN_DBL_BIN_DBL
                               )
       )
order by case test_type
           when 'SINGLE FUNCTION' then 1
           when 'NESTED FUNCTION' then 2
           when 'PTF'             then 3
           else                        9
         end
,        case function_complexity
           when 'NO-OP'  then  1
           when 'SIMPLE' then 2
           when 'SIMPLE' then 3
           else               9
         end
,        case secondary_function_type
           when 'NONE'    then 1
           when 'REGULAR' then 2
           when 'UDF'     then 3
           when 'WITH'    then 4
           else                9
         end
,        case primary_function_type
           when 'NONE'    then 1
           when 'REGULAR' then 2
           when 'UDF'     then 3
           when 'WITH'    then 4
           else                9
         end
;
