-- ######################################################################
-- Dropping objects from a previous run , if they exist
-- ######################################################################

prompt ======================================================================
prompt drop objects from previous runs if they exist
prompt ======================================================================
declare
  cn_object_type_package    constant  all_objects.object_type%type     := 'Package'  ;
  cn_object_type_table      constant  all_objects.object_type%type     := 'Table'    ;
  cn_object_type_procedure  constant  all_objects.object_type%type     := 'Procedure';
  cn_object_type_view       constant  all_objects.object_type%type     := 'View'     ;
  cn_object_type_function   constant  all_objects.object_type%type     := 'Function' ;
  cn_object_type_type       constant  all_objects.object_type%type     := 'Type'     ;


  procedure remove
    (p_type  in  all_objects.object_type%type
    ,p_name  in  all_objects.object_name%type
    )
    is

      e_missing_table   exception;
      e_missing_object  exception;

      pragma exception_init (e_missing_table , -942);    -- ORA-00942: table or view does not exist
      pragma exception_init (e_missing_object,-4043);    -- ORA-04043: object [...] does not exist

    begin   -- remove

      execute immediate 'drop '||p_type||' '||p_name||case p_type
                                                        when cn_object_type_table
                                                        then ' purge'
                                                      end;
      dbms_output.put_line ('Pre-Existing '||p_type||' '||p_name||' dropped!');

    exception
      when e_missing_table
        or e_missing_object
      then
        dbms_output.put_line ('No Pre-Exisiting '||p_type||' '||p_name||' found!');

    end remove;
begin
  remove (cn_object_type_package  , 'ero_test_plsql_performance' );
  remove (cn_object_type_table    , 'ero_test_mem_log'           );
  remove (cn_object_type_table    , 'ero_test_plsql_input'       );
  remove (cn_object_type_table    , 'ero_test_plsql_results'     );
  remove (cn_object_type_table    , 'ero_test_plsql_tests'       );
  
  -- drop objects that may have been created and not dropped due to a failed test run
  remove (cn_object_type_procedure, 'ero_test_procedure');
  remove (cn_object_type_view     , 'ero_test_view'              );
  remove (cn_object_type_function , 'ero_test_function_secondary');
  remove (cn_object_type_function , 'ero_test_function_primary'  );
  remove (cn_object_type_type     , 'ero_test_ntt'               );
  remove (cn_object_type_type     , 'ero_test_ot'                );
  
end;
/
