-- Example of use of plsql in a subquery

prompt Drop test table if it already exists
declare
  procedure drop_table
    (pi_table_name  in  all_tables.table_name%type
    )
    is
      e_missing_table   exception;
      pragma exception_init (e_missing_table,-942);
    begin   -- drop_table
      execute immediate 'drop table '||pi_table_name;
      dbms_output.put_line ('Pre-Existing Table '||pi_table_name||' dropped!');
    exception
      when e_missing_table
      then
        dbms_output.put_line ('No Pre-Exisiting Table '||pi_table_name||' found!');
    end drop_table;
begin
  drop_table ('ero_test_plsql_in_update');
end;
/

prompt create a test table 
create table ero_test_plsql_in_update
as
select floor(dbms_random.value(low => 1000, high => 2000)) input_value
,      cast (null as integer)                              output_value
from   dual
connect by level <= 10
;

prompt List data in test table
select * 
from   ero_test_plsql_in_update
order by input_value
;

prompt Plsql in subquery without WITH_PLSQL hint
prompt This will cause an exception to be raised
update ero_test_plsql_in_update   tst
set    tst.output_value = (with 
                             function digit_sum
                               (pi_integer  in  integer
                               )
                               return integer
                               is
                                 c_as_string  constant  varchar2(15) := to_char(pi_integer,'fm999999999999999');
                                 l_result     integer  := 0;
                               begin
                                 for i_pos in 1 .. length(c_as_string)
                                 loop
                                   l_result := l_result + to_number(substr(c_as_string, i_pos, 1));
                                 end loop;
                                 return (l_result);
                               end;
                           select digit_sum (tst.input_value)    SumOfDigits
                           from   dual
                          )
/


prompt List data in test table
select * 
from   ero_test_plsql_in_update
order by input_value
;

rollback;

prompt Same query with WITH_PLSQL hint
prompt This will execute succesfully
update --+ with_plsql
       ero_test_plsql_in_update   tst
set    tst.output_value = (with 
                             function digit_sum
                               (pi_integer  in  integer
                               )
                               return integer
                               is
                                 c_as_string  constant  varchar2(15) := to_char(pi_integer,'fm999999999999999');
                                 l_result     integer  := 0;
                               begin
                                 for i_pos in 1 .. length(c_as_string)
                                 loop
                                   l_result := l_result + to_number(substr(c_as_string, i_pos, 1));
                                 end loop;
                                 return (l_result);
                               end;
                           select digit_sum (tst.input_value)    SumOfDigits
                           from   dual
                          )
/

prompt List data in test table
select * 
from   ero_test_plsql_in_update
order by input_value
;

prompt drop test table
drop table ero_test_plsql_in_update;

